@@ -23,6 +23,7 @@
 */
 
 
+#define _DEFAULT_SOURCE 1  /* New name for SVID & BSD source defines */
 #define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
 #define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
@@ -1030,7 +1031,7 @@ pixelReader_getbits(pixelReader * const rdrP,
             unsigned long const bitsToTakeMask = lsbmask[nBitsToTake];
                 /* E.g. if nbitsToTake is 4, this is 0x0000000F */
 
-            unsigned int bitsToTake;
+            unsigned long bitsToTake;
                 /* The actual bits we take, in the 'nBitsToTake' low bits */
 
             assert(nBitsToTake <= 32);
